<?php

namespace Netzmacher\Browser\Utility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for rendering a HTML page by TCPDF methods
 *
 * @package TYPO3
 * @subpackage org
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 8.0.12
 * @since 8.0.12
 */
class FAL
{

	/**
	 * Current field
	 *
	 * @var string
	 */
	private $_field = null;

	/**
	 * Current table
	 *
	 * @var string
	 */
	private $_table = null;

	/**
	 * Current uid
	 *
	 * @var integer
	 */
	private $_uid = null;

	/**
	 * getField(): 
	 *
	 * @return	string $value
	 * @access private
	 * @version 8.0.12
	 * @since 8.0.12
	 */
	private function getField()
	{
		if( !empty( $this->_field ) )
		{
			return $this->_field;
		}

		$prompt = 'FATAL ERROR: '
						. '$this->_field is empty. '
						. 'Browser - TYPO3 without PHP'
		;
		var_dump( __METHOD__, __LINE__, $prompt );
		die();
	}

	/**
	 * getTable(): 
	 *
	 * @return	string $value
	 * @access private
	 * @version 8.0.12
	 * @since 8.0.12
	 */
	private function getTable()
	{
		if( !empty( $this->_table ) )
		{
			return $this->_table;
		}

		$prompt = 'FATAL ERROR: '
						. '$this->_table is empty. '
						. 'Browser - TYPO3 without PHP'
		;
		var_dump( __METHOD__, __LINE__, $prompt );
		die();
	}

	/**
	 * getUid(): 
	 *
	 * @return	string $value
	 * @access private
	 * @version 8.0.12
	 * @since 8.0.12
	 */
	private function getUid()
	{
		if( !empty( $this->_uid ) )
		{
			return $this->_uid;
		}

		// #i0312, 170209, dwildt, 1-/+
		//$prompt = 'ERROR: '
		$prompt = 'ERROR: ' . PHP_EOL
						. '$this->_uid is empty. ' . PHP_EOL
						. 'table: "' . $this->_table . '". ' . PHP_EOL
						. 'field: "' . $this->_field . '". ' . PHP_EOL
						. '' . PHP_EOL
						. 'Please check your SQL query. ' . $this->_table . '.uid should be part of the query.' . PHP_EOL
						. '' . PHP_EOL
						. 'Browser - TYPO3 without PHP'
		;
		var_dump( __METHOD__, __LINE__, $prompt );
		// #i0312, 170209, dwildt, 1-
		die();
	}

	/**
	 * getCsvPaths(): 
	 *
	 * @param string $field
	 * @param string $table
	 * @param string $uid
	 * @return	string		$csvPaths
	 * @access public
	 * @internal #t0933
	 * @link https://docs.typo3.org/typo3cms/FileAbstractionLayerReference/UsingFal/Examples/FileFolder/Index.html#getting-referenced-files
	 * @version 8.0.12
	 * @since 8.0.0
	 */
	public function getCsvPaths( $table, $field, $uid )
	{
		$this->setField( $field );
		$this->setTable( $table );
		$this->setUid( $uid );

		$fileRepository = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( 'TYPO3\\CMS\\Core\\Resource\\FileRepository' );
		try
		{
			// #i0312, 170209, dwildt, ~
			$getUid = $this->getUid();
			if( empty( $getUid ) )
			{
				return;
			}
			$fileObjects = $fileRepository->findByRelation( $this->getTable(), $this->getField(), $getUid );
		}
		catch( \Exception $e )
		{
			$prompt = ''
							. 'FATAL ERROR: '
							. $e->getMessage() . ' at '
							. __METHOD__ . ' (#' . __LINE__ . '). '
							. 'Sorry. Browser - TYPO3 without PHP.'
			;
			die( $prompt );
		}

		// get Imageobject information
		$aPaths = array();
		foreach( $fileObjects as $value )
		{
			$aPaths[] = $value->getPublicUrl();
		}
		$csvPaths = implode( ',', $aPaths );

//		var_dump( __METHOD__, __LINE__, $this->getTable(), $this->getField(), $this->getUid(), $csvPaths );
		return $csvPaths;
	}

	/**
	 * setField(): 
	 *
	 * @param string $value
	 * @return	void
	 * @access private
	 * @version 8.0.12
	 * @since 8.0.12
	 */
	private function setField( $value )
	{
		$this->_field = $value;
	}

	/**
	 * setTable(): 
	 *
	 * @param string $value
	 * @return	void
	 * @access private
	 * @version 8.0.12
	 * @since 8.0.12
	 */
	private function setTable( $value )
	{
		$this->_table = $value;
	}

	/**
	 * setUid(): 
	 *
	 * @param integer $value
	 * @return	void
	 * @access private
	 * @version 8.0.12
	 * @since 8.0.12
	 */
	private function setUid( $value )
	{
		$this->_uid = $value;
	}

}
